@@ -224,8 +224,8 @@ class PyCBCHDFInjFindExecutable(Executable):
         if veto_name is not None:
             node.add_input_opt('--veto-file', veto_file)
         node.add_opt('--segment-name', veto_name)
-        node.new_output_file_opt(inj_xml_file[0].segment, '.hdf', '--output-file',
-                                 tags=tags)
+        node.new_output_file_opt(inj_xml_file[0].segment, '.hdf',
+                                 '--output-file', tags=tags)
         return node
 
 class PyCBCDistributeBackgroundBins(Executable):
@@ -276,9 +276,13 @@ class PyCBCMultiifoAddStatmap(PyCBCMultiifoCombineStatmap):
 
     current_retention_level = Executable.MERGED_TRIGGERS
     def create_node(self, statmap_files, background_files, tags=None):
+        if tags is None:
+            tags = []
         node = super(PyCBCMultiifoAddStatmap, self).create_node(statmap_files,
                                                             tags=tags)
-        if 'injections' in tags:
+        # Enforce upper case
+        ctags = [t.upper() for t in (tags + self.tags)]
+        if 'INJECTIONS' in ctags:
             node.add_input_list_opt('--background-files', background_files)
 
         return node
@@ -334,24 +338,29 @@ def merge_single_detector_hdf_files(workflow, bank_file, trigger_files, out_dir,
         out += node.output_files
     return out
 
-def setup_trigger_fitting(workflow, insps, hdfbank, veto_file, veto_name):
+def setup_trigger_fitting(workflow, insps, hdfbank, veto_file, veto_name,
+                          output_dir=None, tags=None):
     if not workflow.cp.has_option('workflow-coincidence', 'do-trigger-fitting'):
         return FileList()
     else:
-        assert len(hdfbank) == 1  # must be a list with exactly 1 bank file
-        assert len(veto_file) == 1
-        assert len(veto_name) == 1
         smoothed_fit_files = FileList()
         for i in workflow.ifos:
             ifo_insp = [insp for insp in insps if (insp.ifo == i)]
             assert len(ifo_insp)==1
-            raw_node = PyCBCFitByTemplateExecutable(workflow.cp,
-                'fit_by_template', ifos=i).create_node(ifo_insp[0], hdfbank[0],
-                                                    veto_file[0], veto_name[0])
+            ifo_insp = ifo_insp[0]
+            raw_exe = PyCBCFitByTemplateExecutable(workflow.cp,
+                                                   'fit_by_template', ifos=i,
+                                                   out_dir=output_dir,
+                                                   tags=tags)
+            raw_node = raw_exe.create_node(ifo_insp, hdfbank,
+                                           veto_file, veto_name)
             workflow += raw_node
-            smooth_node = PyCBCFitOverParamExecutable(workflow.cp,
-                'fit_over_param', ifos=i).create_node(raw_node.output_files[0],
-                                                                    hdfbank[0])
+            smooth_exe = PyCBCFitOverParamExecutable(workflow.cp,
+                                                     'fit_over_param', ifos=i,
+                                                     out_dir=output_dir,
+                                                     tags=tags)
+            smooth_node = smooth_exe.create_node(raw_node.output_file,
+                                                 hdfbank)
             workflow += smooth_node
             smoothed_fit_files += smooth_node.output_files
         return smoothed_fit_files
@@ -364,7 +373,7 @@ def find_injections_in_hdf_coinc(workflow, inj_coinc_file, inj_xml_file,
     exe = PyCBCHDFInjFindExecutable(workflow.cp, 'hdfinjfind',
                                     ifos=workflow.ifos,
                                     out_dir=out_dir, tags=tags)
-    node = exe.create_node(inj_coinc_file, inj_xml_file, veto_file, veto_name, tags)
+    node = exe.create_node(inj_coinc_file, inj_xml_file, veto_file, veto_name)
     workflow += node
     return node.output_files[0]
 
@@ -414,9 +423,10 @@ def setup_multiifo_statmap(workflow, ifos, coinc_files, out_dir, tags=None):
     ifolist = ' '.join(ifos)
     stat_node = statmap_exe.create_node(coinc_files, ifolist)
     workflow.add_node(stat_node)
-    return stat_node.output_files[0], stat_node.output_files
+    return stat_node.output_file
 
-def setup_multiifo_statmap_inj(workflow, ifos, coinc_files, background_file, out_dir, tags=None):
+def setup_multiifo_statmap_inj(workflow, ifos, coinc_files, background_file,
+                               out_dir, tags=None):
     tags = [] if tags is None else tags
 
     statmap_exe = PyCBCMultiifoStatMapInjExecutable(workflow.cp,
@@ -425,8 +435,11 @@ def setup_multiifo_statmap_inj(workflow, ifos, coinc_files, background_file, out
                                                     tags=tags, out_dir=out_dir)
 
     ifolist = ' '.join(ifos)
-    stat_node = statmap_exe.create_node(FileList(coinc_files['injinj']), background_file,
-                                     FileList(coinc_files['injfull']), FileList(coinc_files['fullinj']), ifolist)
+    stat_node = statmap_exe.create_node(FileList(coinc_files['injinj']),
+                                        background_file,
+                                        FileList(coinc_files['injfull']),
+                                        FileList(coinc_files['fullinj']),
+                                        ifolist)
     workflow.add_node(stat_node)
     return stat_node.output_files[0]
 
@@ -478,19 +491,25 @@ def setup_background_bins(workflow, coinc_files, bank_file, out_dir, tags=None):
 
     return cstat_node.output_files[0], statmap_files
 
-def setup_statmap_inj(workflow, coinc_files, background_file, bank_file, out_dir, tags=None):
+def setup_statmap_inj(workflow, coinc_files, background_file, bank_file,
+                      out_dir, tags=None):
     tags = [] if tags is None else tags
-    if workflow.cp.has_option_tags('workflow-coincidence', 'background-bins', tags):
-        return setup_background_bins_inj(workflow, coinc_files, background_file, bank_file, out_dir, tags=tags)
+    if workflow.cp.has_option_tags('workflow-coincidence',
+                                   'background-bins', tags):
+        return setup_background_bins_inj(workflow, coinc_files,
+                                         background_file, bank_file, out_dir,
+                                         tags=tags)
     else:
-        return setup_simple_statmap_inj(workflow, coinc_files, background_file, out_dir, tags=tags)
+        return setup_simple_statmap_inj(workflow, coinc_files, background_file,
+                                        out_dir, tags=tags)
 
-def setup_simple_statmap_inj(workflow, coinc_files, background_file, out_dir, tags=None):
+def setup_simple_statmap_inj(workflow, coinc_files, background_file, out_dir,
+                             tags=None):
     tags = [] if tags is None else tags
 
     statmap_exe = PyCBCStatMapInjExecutable(workflow.cp, 'statmap_inj',
-                                              ifos=workflow.ifos,
-                                              tags=tags, out_dir=out_dir)
+                                            ifos=workflow.ifos,
+                                            tags=tags, out_dir=out_dir)
 
     stat_node = statmap_exe.create_node(FileList(coinc_files['injinj']),
                                         background_file,
@@ -531,11 +550,12 @@ def setup_background_bins_inj(workflow, coinc_files, background_file,
     statmap_files = FileList([])
     for i in range(len(background_bins)):
         statnode = statmap_exe.create_node(
-                                   FileList([coinc_files['injinj'][i]]),
-                                   FileList([background_file[i]]),
-                                   FileList([coinc_files['injfull'][i]]),
-                                   FileList([coinc_files['fullinj'][i]]),
-                                   tags=['BIN_%s' % i])
+            FileList([coinc_files['injinj'][i]]),
+            FileList([background_file[i]]),
+            FileList([coinc_files['injfull'][i]]),
+            FileList([coinc_files['fullinj'][i]]),
+            tags=['BIN_%s' % i]
+        )
         workflow += statnode
         statmap_files.append(statnode.output_files[0])
 
@@ -544,11 +564,13 @@ def setup_background_bins_inj(workflow, coinc_files, background_file,
 
     return cstat_node.output_files[0]
 
-def setup_interval_coinc_inj(workflow, hdfbank, full_data_trig_files, inj_trig_files,
-              stat_files, background_file, veto_file, veto_name, out_dir, tags=None):
+def setup_interval_coinc_inj(workflow, hdfbank, full_data_trig_files,
+                             inj_trig_files, stat_files, background_file,
+                             veto_file, veto_name, out_dir, tags=None):
     """
-    This function sets up exact match coincidence and background estimation
+    Set up exact match coincidence and background estimation
 
+    This function sets up exact match coincidence and background estimation
     using a folded interval technique.
     """
     if tags is None:
@@ -562,7 +584,8 @@ def setup_interval_coinc_inj(workflow, hdfbank, full_data_trig_files, inj_trig_f
     hdfbank = hdfbank[0]
 
     if len(workflow.ifos) > 2:
-        raise ValueError('This coincidence method only supports two-ifo searches')
+        err_msg = 'This coincidence method only supports two-ifo searches'
+        raise ValueError(err_msg)
 
     # Wall time knob and memory knob
     factor = int(workflow.cp.get_opt_tags('workflow-coincidence',
@@ -593,7 +616,7 @@ def setup_interval_coinc_inj(workflow, hdfbank, full_data_trig_files, inj_trig_f
                                                    stat_files,
                                                    veto_file, veto_name,
                                                    group_str,
-                                                   tags=[str(i)])
+                                                   tags=["JOB"+str(i)])
             bg_files[ctag] += coinc_node.output_files
             workflow.add_node(coinc_node)
 
@@ -660,11 +683,6 @@ def setup_multiifo_interval_coinc_inj(workflow, hdfbank, full_data_trig_files,
     make_analysis_dir(out_dir)
     logging.info('Setting up coincidence for injections')
 
-    if len(hdfbank) != 1:
-        raise ValueError('Must use exactly 1 bank file for this coincidence '
-                         'method, I got %i !' % len(hdfbank))
-    hdfbank = hdfbank[0]
-
     # Wall time knob and memory knob
     factor = int(workflow.cp.get_opt_tags('workflow-coincidence',
                                           'parallelization-factor', tags))
@@ -711,7 +729,7 @@ def setup_multiifo_interval_coinc_inj(workflow, hdfbank, full_data_trig_files,
                                                    group_str,
                                                    pivot_ifo,
                                                    fixed_ifo,
-                                                   tags=[veto_name, str(i)])
+                                                   tags=['JOB'+str(i)])
 
             bg_files[ctag] += coinc_node.output_files
             workflow.add_node(coinc_node)
@@ -723,7 +741,7 @@ def setup_multiifo_interval_coinc_inj(workflow, hdfbank, full_data_trig_files,
                                       tags=tags + [veto_name])
 
 def setup_multiifo_interval_coinc(workflow, hdfbank, trig_files, stat_files,
-                                  veto_files, veto_names, out_dir, pivot_ifo,
+                                  veto_file, veto_name, out_dir, pivot_ifo,
                                   fixed_ifo, tags=None):
     """
     This function sets up exact match multiifo coincidence
@@ -733,11 +751,6 @@ def setup_multiifo_interval_coinc(workflow, hdfbank, trig_files, stat_files,
     make_analysis_dir(out_dir)
     logging.info('Setting up coincidence')
 
-    if len(hdfbank) != 1:
-        raise ValueError('Must use exactly 1 bank file for this coincidence '
-                         'method, I got %i !' % len(hdfbank))
-    hdfbank = hdfbank[0]
-
     ifos, _ = trig_files.categorize_by_attr('ifo')
     findcoinc_exe = PyCBCFindMultiifoCoincExecutable(workflow.cp, 'multiifo_coinc',
                                              ifos=ifos,
@@ -745,26 +758,26 @@ def setup_multiifo_interval_coinc(workflow, hdfbank, trig_files, stat_files,
 
     # Wall time knob and memory knob
     factor = int(workflow.cp.get_opt_tags('workflow-coincidence',
-                                          'parallelization-factor', tags))
+                                          'parallelization-factor',
+                                          [findcoinc_exe.ifo_string] + tags))
 
     statmap_files = []
-    for veto_file, veto_name in zip(veto_files, veto_names):
-        bg_files = FileList()
-        for i in range(factor):
-            group_str = '%s/%s' % (i, factor)
-            coinc_node = findcoinc_exe.create_node(trig_files, hdfbank,
-                                                   stat_files,
-                                                   veto_file, veto_name,
-                                                   group_str,
-                                                   pivot_ifo,
-                                                   fixed_ifo,
-                                                   tags=[veto_name, str(i)])
-            bg_files += coinc_node.output_files
-            workflow.add_node(coinc_node)
+    bg_files = FileList()
+    for i in range(factor):
+        group_str = '%s/%s' % (i, factor)
+        coinc_node = findcoinc_exe.create_node(trig_files, hdfbank,
+                                               stat_files,
+                                               veto_file, veto_name,
+                                               group_str,
+                                               pivot_ifo,
+                                               fixed_ifo,
+                                               tags=['JOB'+str(i)])
+        bg_files += coinc_node.output_files
+        workflow.add_node(coinc_node)
 
-        statmap_files += [setup_multiifo_statmap(workflow, ifos, bg_files,
-                                                 out_dir,
-                                                 tags=tags + [veto_name])]
+    statmap_files = setup_multiifo_statmap(workflow, ifos, bg_files,
+                                           out_dir,
+                                           tags=tags)
 
     logging.info('...leaving coincidence ')
     return statmap_files
@@ -818,12 +831,10 @@ def setup_multiifo_combine_statmap(workflow, final_bg_file_list, bg_file_list,
                             tags=tags, out_dir=out_dir)
 
     if cstat_exe_name == 'pycbc_multiifo_combine_statmap':
-        combine_statmap_node = cstat_exe.create_node(final_bg_file_list,
-                                                     tags=tags)
+        combine_statmap_node = cstat_exe.create_node(final_bg_file_list)
     elif cstat_exe_name == 'pycbc_multiifo_add_statmap':
         combine_statmap_node = cstat_exe.create_node(final_bg_file_list,
-                                                     bg_file_list,
-                                                     tags=tags)
+                                                     bg_file_list)
 
     workflow.add_node(combine_statmap_node)
     return combine_statmap_node.output_file
@@ -848,9 +859,13 @@ def setup_multiifo_exclude_zerolag(workflow, statmap_file, other_statmap_files,
     workflow.add_node(exc_zerolag_node)
     return exc_zerolag_node.output_file
 
-def rerank_coinc_followup(workflow, statmap_file, bank_file, out_dir, tags,
+def rerank_coinc_followup(workflow, statmap_file, bank_file, out_dir,
+                          tags=None,
                           injection_file=None,
                           ranking_file=None):
+    if tags is None:
+        tags = []
+
     make_analysis_dir(out_dir)
 
     if not workflow.cp.has_section("workflow-rerank"):
@@ -864,14 +879,14 @@ def rerank_coinc_followup(workflow, statmap_file, bank_file, out_dir, tags,
     for ifo in workflow.ifos:
         make_analysis_dir('strain_files')
         node = Executable(workflow.cp, 'strain_data_reduce', ifos=[ifo],
-                          out_dir='strain_files').create_node()
+                          out_dir='strain_files', tags=tags).create_node()
         node.add_opt('--gps-start-time', workflow.analysis_time[0])
         node.add_opt('--gps-end-time', workflow.analysis_time[1])
         if injection_file:
             node.add_input_opt('--injection-file', injection_file)
 
         fil = node.new_output_file_opt(workflow.analysis_time, '.hdf',
-                                       '--output-file', tags=tags)
+                                       '--output-file')
         stores.append(fil)
         workflow += node
 
@@ -881,7 +896,7 @@ def rerank_coinc_followup(workflow, statmap_file, bank_file, out_dir, tags,
     node.add_input_opt('--statmap-file', statmap_file)
     node.add_input_opt('--bank-file', bank_file)
     trigfil = node.new_output_file_opt(workflow.analysis_time, '.hdf',
-                                   '--output-file', tags=tags)
+                                   '--output-file')
     workflow += node
 
     # Parallelize coinc trigger followup
@@ -915,4 +930,4 @@ def rerank_coinc_followup(workflow, statmap_file, bank_file, out_dir, tags,
     node.new_output_file_opt(workflow.analysis_time, '.hdf',
                              '--output-file')
     workflow += node
-    return node.output_files[0]
+    return node.output_file
